require "trade-o-matic/version"
require 'trade-o-matic/standard'
require 'trade-o-matic/command'
require 'trade-o-matic/errors'

require 'trade-o-matic/structs/currency'
require 'trade-o-matic/structs/currency_pair'
require 'trade-o-matic/structs/converter'
require 'trade-o-matic/structs/price'
require 'trade-o-matic/structs/order'
require 'trade-o-matic/structs/transaction'
require 'trade-o-matic/structs/slope'
require 'trade-o-matic/structs/ask_slope'
require 'trade-o-matic/structs/bid_slope'
require 'trade-o-matic/structs/book'

require 'trade-o-matic/services/backend_factory'

require 'trade-o-matic/core/market_loader'
require 'trade-o-matic/core/exchange'
require 'trade-o-matic/core/market'
require 'trade-o-matic/core/account'
require 'trade-o-matic/core/account_proxy'
require 'trade-o-matic/core/account_order'
require 'trade-o-matic/core/balance'
require 'trade-o-matic/core/transaction_event'

require 'trade-o-matic/endpoints/base_endpoint'
require 'trade-o-matic/endpoints/bitcoin_address'

require 'trade-o-matic/adapters/base_backend'

require 'trade-o-matic/converters/fixed_converter'
require 'trade-o-matic/converters/sync_converter'
require 'trade-o-matic/converters/inverse_converter'
require 'trade-o-matic/converters/compound_converter'

module Trader
  # Default conversions
  Currency.register_conversion :BTC, :SATOSHI, 100_000_000
  Currency.register_conversion :SATOSHI, :BTC, 0.00000001

  def self.exchange(_backend)
    Exchange.new BackendFactory.build(_backend)
  end

  def self.market(_backend, _pair, _quote=nil)
    exchange(_backend).market_for(_pair, _quote)
  end

  def self.account(_backend, _credentials)
    exchange(_backend).login(_credentials)
  end

  def self.setup_game_backend(&_block)
    require 'trade-o-matic/adapters/game_backend'
    GameBackend::Configuration.tap &_block
    nil
  end

  def self.game
    require 'trade-o-matic/adapters/game_backend'
    GameBackend.new
  end
end
