require 'trade-o-matic/adapters/base/raw_account_order'
require 'trade-o-matic/adapters/base/raw_balance'

module Trader
  class BaseBackend
    attr_reader :name

    def initialize(_name)
      @name = _name
    end

    def get_available_markets
      []
    end

    def fill_book(_book)
      not_supported :book
    end

    def get_session(_credentials)
      _credentials
    end

    def get_balance(_session, _currency)
      not_supported :get_balance
    end

    def generate_endpoint(_session, _currency)
      not_supported :deposit
    end

    def withdraw_to_endpoint(_session, _currency, _amount, _endpoint)
      not_supported :withdraw
    end

    def get_orders(_session, _pair)
      not_supported :list_orders
    end

    def create_order(_session, _pair, _volume, _price, _type)
      not_supported :create_order
    end

    def fetch_order(_session, _id)
      not_supported :fetch_order
    end

    def cancel_order(_session, _id)
      not_supported :cancel_order
    end

    def listen_transactions(_pair, &_block)
      not_supported :listen_transactions
    end

    def not_supported(_feature)
      raise NotSupportedError.new name, _feature
    end

    private

    def error(_messsage)
      raise BackendError.new name, _messsage
    end
  end
end