module Trader
  class GameBackend
    class CancelOrder < Command.new(:state, :account, :order_id)
      def perform
        raise ArgumentError, 'order belong to another account' if order['account'] != account
        raise ArgumentError, 'order already closed' if order['status'] != 'open'

        market['open'].delete order
        market['closed'] << order
        order['status'] = 'canceled'

        base_balance = state.balance_for account, market['base']
        quote_balance = state.balance_for account, market['quote']

        if bid?
          quote = SFM.quote order['volume'], order['limit']
          quote_balance['available'] += quote
          quote_balance['frozen'] -= quote
        else
          base_balance['available'] += order['volume']
          base_balance['frozen'] -= order['volume']
        end

        GameOrder.new order
      end

      private

      def bid?
        order['instruction'] == 'bid'
      end

      def order
        result[:order]
      end

      def market
        result[:market]
      end

      def result
        @result ||= state.find_order_by_id order_id
      end
    end
  end
end