require 'yaml'

module Trader
  class GameBackend
    class State
      def initialize
        reset
      end

      def load_from_file(_path)
        if File.exists? _path
          @state = YAML.load File.read(_path)
        else
          reset
        end
      end

      def store_to_file(_path)
        File.write _path, YAML.dump(@state)
      end

      def all_markets
        state['markets'].values.map { |m| CurrencyPair.new(m['base'], m['quote']) }
      end

      def unique_id
        state['next_id'] += 1
      end

      def account_for(_account_name, create: true)
        account = state['accounts'][_account_name]
        account = state['accounts'][_account_name] = {} if account.nil? && create
        account
      end

      def balance_for(_account_name, _currency, create: true)
        account = account_for(_account_name)
        balance = account[_currency.to_s]
        balance = account[_currency.to_s] = new_balance if balance.nil? && create
        balance
      end

      def market_for(_pair, create: true)
        market = state['markets'][_pair.to_s]
        market = state['markets'][_pair.to_s] = new_market_for(_pair) if market.nil? && create
        market
      end

      def find_order_by_id(_id)
        state['markets'].values.each do |market|
          order = market['open'].find { |o| o['id'] == _id } ||
            market['closed'].find { |o| o['id'] == _id }
          return { market: market, order: order } unless order.nil?
        end
        raise ArgumentError, 'order id not found'
      end

      private

      attr_reader :state

      def new_market_for(_pair)
        {
          'base' => _pair.base.to_s,
          'quote' => _pair.quote.to_s,
          'open' => [],
          'closed' => []
        }
      end

      def new_balance
        { 'available' => 0, 'frozen' => 0 }
      end

      def reset
        @state = {
          'next_id' => 0,
          'accounts' => {},
          'markets' => {}
        }
      end
    end
  end
end
