module Trader
  module MarketLoader
    def market_for(_pair, _quote=nil)
      _pair = CurrencyPair.for_code _pair, _quote
      ensure_supported_market _pair
      Market.new(backend, _pair).refresh!
    end

    def has_market_for?(_pair, _quote=nil)
      _pair = CurrencyPair.for_code _pair, _quote
      available_markets.include? _pair
    end

    def available_markets
      @markets ||= backend.get_available_markets
    end

    private

    def ensure_supported_market(_pair)
      backend.not_supported _pair unless has_market_for? _pair
    end

    def ensure_supported_currency(_currency)
      unless available_markets.any? { |m| m.quote == _currency || m.base == _currency }
        backend.not_supported _currency
      end
    end
  end
end