module Trader
  class TransactionEvent
    def initialize(_raw, _forced_pair = nil)
      @raw = _raw
      @forced_pair = _forced_pair
    end

    def convert_to(_pair, _quote=nil)
      forced_pair = CurrencyPair.for_code _pair, _quote
      return self if forced_pair == pair
      self.class.new @raw, forced_pair
    end

    def pair
      forced_pair || original_pair
    end

    def original_pair
      @raw[:pair]
    end

    def timestamp
      @raw[:ts]
    end

    def direction
      @raw[:direction]
    end

    def price
      convert_quote original_pair.quote.pack @raw[:price]
    end

    def volume
      convert_base original_pair.quote.pack @raw[:volume]
    end

    private

    attr_reader :forced_pair

    def convert_base(_price)
      return _price if forced_pair.nil?
      _price.convert_to forced_pair.base
    end

    def convert_quote(_price)
      return _price if forced_pair.nil?
      _price.convert_to forced_pair.quote
    end
  end
end